import moment from 'moment';

const getLapse = (from, to) => {
  return (to - from) / 60 / 60;
};

const getInProgressTime = (from) => {
  return ((new Date().getTime() / 1000) - from) / 60 / 60;
};

const getTotalHours = (timesheets) => timesheets.reduce((acc, ts) => {
  if (!ts.IsLeave) {
    if (ts.IsInProgress) {
      const nowSeconds = (new Date().getTime() / 1000);
      const startTime = new Date(ts.StartTimeLocalized).getTime() / 1000;
      const slotsTime = getSlotsTime(ts, nowSeconds);
      const inprogresstime = getLapse(startTime, nowSeconds);
      // console.log('inprogresstime, slotsTime :>> ', ts.Id, inprogresstime, slotsTime);
      // console.log('in progreess ', acc);
      acc += (inprogresstime - slotsTime);
      // console.log('in progreess ', acc);
    } else {
      acc += ts.TotalTime
    }
  }
  return acc;
}, 0);

const getSlotsTime = (timesheet, now) => {
  return (timesheet.Slots || []).reduce((sum, slot) => {
    const start = slot.intUnixStart;
    const end = slot.intUnixEnd;
    const isInProgress = slot.strState === 'In Progress';
    const isFinished = slot.strState === 'Finished';

    let toAdd = 0;
    if (isInProgress) {
      toAdd = getLapse(start, now);
    } else if (isFinished) {
      toAdd = getLapse(start, end);
    }
    // console.log('timesheet :>> ', timesheet);
    // console.log('start, end :>> ', start, end);
    // console.log('getLapse(start, end) :>> ', getLapse(start, end));
    // console.log('isInProgress :>> ', isInProgress);
    // const toAdd = !isInProgress ? getLapse(start, end) : getInProgressTime(start);
    // console.log('toAdd :>>', toAdd);
    return sum += toAdd;
  }, 0);
}

const getTotalLeaveHours = (timesheets) => timesheets.reduce((acc, ts) => {
  if (ts.IsLeave) {
    acc += ts.TotalTime
  }
  return acc;
}, 0);

const getEmployeeClasses = (hours, isInProgress) => {
  const classes = [];
  if (hours >= 38) {
    classes.push('red');
  } else if (hours >= 30) {
    classes.push('yellow');
  } else {
    classes.push('green');
  }
  if (isInProgress) {
    classes.push('in-progress');
  }

  return classes.join(' ');
}

const leftPadding = (val, y = 2) => {
  const exp = (10 ** y) * (val < 0 ? -1 : 1);
  const v = ((exp + val) + '');
  return val < 0 ? '-' + v.substring(2) : v.substring(1);
};

const formatTime = (val) => {
  const str = Number(val).toFixed(2).split('.');
  const hours = Number(str[0]);
  const mins = Math.round((Number('0.' + (str[1] || 0)) * 60));
  return {
    hours: leftPadding(hours),
    minutes: leftPadding(mins)
  };
}

const isLunch = ts => {
  for (let index = 0; index < ts.Slots.length; index++) {
    const slot = ts.Slots[index];
    if (slot.strState === 'In Progress' && slot.strType === 'B') {
      return true;
    }
  }

  return false;
}

export const getEmployeeInfo = (raw) => {
  const last = raw[raw.length - 1];
  const totalHours = getTotalHours(raw);
  const otHours = totalHours - 40;
  const metadata = last._DPMetaData;
  const operationalUnit = metadata.OperationalUnitInfo || {};
  const lunch = isLunch(last);
  // console.log('isLunch :>> ', last.Id, lunch, metadata.EmployeeInfo.DisplayName);
  return {
    displayName: metadata.EmployeeInfo.DisplayName,
    unit: operationalUnit.OperationalUnitName || '',
    site: operationalUnit.CompanyName || '',
    inProgress: hasInProgress(raw),
    totalTimesheets: raw.length,
    totalHours,
    totalHoursObject: formatTime(totalHours),
    employeeClasses: getEmployeeClasses(totalHours, last.IsInProgress),
    locationClasses: getEmployeeLocationClass(operationalUnit.CompanyName),
    overtime: formatTime(otHours),
    isLunch: lunch,
  };
}

const hasInProgress = (raw) => {
  return raw.filter(d => d.IsInProgress).length > 0;
}

export const employeeMapper = (data) => {
  const reduced = data
    // .filter(d => {
    //   return d._DPMetaData.EmployeeInfo.DisplayName.indexOf('Bugs') !== -1
    // })
    .reduce((acc, val) => {
      if (!acc[val.Employee]) {
        acc[val.Employee] = [];
      }
      acc[val.Employee].push(val);
      return acc;
    }, {});
  const employees = [];
  for (const employeeId in reduced) {
    if (Object.hasOwnProperty.call(reduced, employeeId)) {
      const timesheets = reduced[employeeId];
      employees.push(getEmployeeInfo(timesheets));
    }
  }

  employees.sort((a, b) => {
    return a.totalHours > b.totalHours ? -1 : 1;
  });

  const active = []
  const inactive = []

  employees.forEach(e => {
    e.inProgress ? active.push(e) : inactive.push(e);
  });

  // console.log('active :>> ', active);

  return active.concat(inactive);
}

function getWeekDelta(cutDay, cutHour, today, hour) {
  // dia actual mayor que dia de corte Tengo que traer desde el jueves misma semana
  // mismo dia despues de la hora traer misma semana
  // dia actual menor que el dia de corte Tengo que traer desde el jueves semana anterior
  // mismo dia antes de la hora traer semana anterior
  let weekDelta = (today < cutDay || (today === cutDay && hour < cutHour)) ? -1 : 0;
  return weekDelta;
}

export function getTime(timezone, cutDay, cutHour) {
  const now = moment().tz(timezone);
  let today = now.day();
  let hour = now.hour();
  let weekDelta = getWeekDelta(cutDay, cutHour, today, hour);

  now
    .day(cutDay)
    .hour(cutHour)
    .minute(0)
    .second(0)
    .millisecond(0)
    .add(weekDelta, 'week');
  // console.log(now.format('HH:mm:ss'), timezone);
  return now;
}

const getEmployeeLocationClass = (site = '') => {
  let k = 'gray';
  switch (site.toUpperCase()) {
    case 'SHADOW MOUNTAIN':
      k = 'pink';
      break;
    case 'NORTH LOOP':
      k = 'blue';
      break;
    case 'MONTANA':
      k = 'white';
      break;
    case 'GATEWAY':
      k = 'green';
      break;
    case 'REMOTE':
      k = 'red';
      break;
    case 'ADMINISTRATION':
      k = 'orange';
      break;

    default:
      break;
  }

  return k;
}